////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//

#ifndef ANISO_MICRO_DIST_H
#define ANISO_MICRO_DIST_H


#include "bxdfmicrofacet.h"
#include "microfacetdistribution.h"

class CAnisotropicMicroDist : public CMicrofacetDistribution
{
public:
  CAnisotropicMicroDist( double X, double Y );
  virtual ~CAnisotropicMicroDist(void);

  double D(const Vector& HalfVec, const Vector& Normal) const {
    // float CosThetah = fabs( CosTheta(HalfVec) );
    double CosThetah = fabs( ( Normal.dot( HalfVec ) ) );
    double e = (m_ExpX * HalfVec.x * HalfVec.x + m_ExpY * HalfVec.y * HalfVec.y) / (1.0 - CosThetah * CosThetah);
    return static_cast<double>( sqrt(( m_ExpX + 2) * (m_ExpY + 2 ) ) / PI * pow( CosThetah, e) );
  }

  virtual void    Sample_f( const Vector& Out, Vector& In, const Vector& Normal, double u1, double u2, float& Pdf ) const;
  virtual float   GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const;
  void            SampleFirstQuadrant(double u1, double u2, double& Phi, double& CosTheta) const;

private:
  double m_ExpX;
  double m_ExpY;
};


#endif
// vim:ts=2
