////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#ifndef MAT_TOON_H
#define MAT_TOON_H

#include "physmatinterface.h"
#include "bxdfbase.h"
#include "bxdftoon.h"
#include "baseintegrator.h"

class CMatToon : public CPhysMatInterface
{
public:
  CMatToon( CBaseIntegrator* pInt );
  virtual ~CMatToon(void);

	// f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
	// Out = ray coming FROM the camera, In = ray going towards the light
	virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade = 1.0 ) {
		UNUSED( Power );
		return m_BxDF.f( Out, In, Normal, RenderMat, LightColor, Shade );
	}

	// By default only diffuse sampling is performed. This function needs to be overwritten by the actual physical materials that
	// have reflective and probably translucent components!
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, CBxDFBase::BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 ) {
		UNUSED( Power );
		return m_BxDF.Sample_f( Out, In, Pdf, Normal, RenderMat, Sampler, Type, LightColor, Shade );
  }

protected:
  CBxDFToon   m_BxDF;
};


#endif
// vim:ts=2
