/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	2012
 *
 */

#ifndef __GRAPH_EXPORTBASE_H
#define __GRAPH_EXPORTBASE_H

#include <vector>
#include <fstream>

#include "os.h"
#include "math/vector2d.h"

typedef std::vector<Vector2D>	Path;

class ExportBase {
protected:
	std::ofstream	_file;
	std::string	_fileName;

protected:
	// prevent copies
	ExportBase( const ExportBase& rhs );
	ExportBase& operator=( const ExportBase& rhs );

public:
	ExportBase(const std::string& filename ) : _fileName(filename)
			{ _file.open(filename.c_str()); };
	virtual ~ExportBase()	{ if (_file) _file.close(); }

	operator bool () const	{ return _file.is_open(); } // overload the boolean conversion op

	// prevent instantiation of base class & enforce minimum exporter features
	virtual bool line(double x1, double y1, double x2, double y2, dword color=0, const char* layer=NULL)=0;
	virtual bool circle(double x, double y, double radius, dword color=0, const char* layer=NULL)=0;
	virtual bool arc(double x, double y, double radius, double startPhi, double endPhi, dword color=0, const char* layer=NULL)=0;
	virtual bool point(double x, double y, dword color=0, const char* layer=NULL) {
				return line(x,y,x,y,color,layer);
			}
	virtual bool polyline(Path& path, dword color=0, const char* layer=NULL) {
				bool b = true;
				for (size_t i=0; i<path.size()-1; i++)
					b &= line(path[i].x,   path[i].y,
						  path[i+1].x, path[i+1].y,
						  color, layer);
				return b;
			}
	virtual bool rectangle(double left, double bottom, double right, double top, dword color=0, const char* layer=NULL) {
				return	line(left, bottom, right, bottom, color, layer) &&
					line(right, bottom, right, top, color, layer)   &&
					line(right, top, left, top, color, layer)       &&
					line(left, top, left, bottom, color, layer);
			}

}; // ExportBase

#endif
