/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
// Authors:	Christian.Theis@cern.ch
//		Vasilis.Vlachoudis@cern.ch
 * Date:	17-Jul-2020
 */

#ifndef __MATH_MORTON_H
#define __MATH_MORTON_H

/* ============================== Morton ============================== */
/** Simple Morton encoder to map 2D coordinates to a space filling Z-curve */
class Morton {
public:
	static bool	next2D(int& x, int& y, const int width, const int height);

	static unsigned encode2D(unsigned x, unsigned y)
			{ return (interleave(y) << 1) + interleave(x); };

#if _DEBUG>0
	static void	test(int W, int H, bool plot);
#endif
protected:
	static unsigned interleave(unsigned x) {
			x &= 0x0000ffff;
			x = (x ^ (x << 8)) & 0x00ff00ff;
			x = (x ^ (x << 4)) & 0x0f0f0f0f;
			x = (x ^ (x << 2)) & 0x33333333;
			x = (x ^ (x << 1)) & 0x55555555;
			return x;
		}
}; // Morton

#endif
