/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#ifndef __PARTICLE_H
#define __PARTICLE_H

#include "os.h"
#include "geometry/geo.h"
#include "geometry/ray.h"
#include "physics/units.h"

/* =============================== Particle ================================= */
class Particle : public Ray {
public:
	enum class Type {
		PNeutron,
		PProton,
		PGamma,
		PElectron,
		PPositron,
		PDeuteron,
		PTriton,
		PAlpha,
		PIon,
		PPionPlus,
		PPionMinus,
	};
public:
//	ParticleType	type;
//	PType	type;			// particle type
//	double	mass;
//	double	energy;

public:
	Particle()	{}
virtual const char*	name()	const = 0;
virtual double		mass()	const = 0;
}; // Particle

/* =============================== Caloron ================================== *
 * Fictitious particle transfering heat
 */
class Caloron : public Particle {
public:
virtual const char*	name()	const { return "Caloron"; }
virtual double		mass()	const { return 0.0; }
} ;

/* =============================== Neutron ================================== */
class Neutron : public Particle {
public:
virtual const char*	name()	const { return "Neutron"; }
virtual double		mass()	const { return Mn; }
}; // Neutron

#endif
