/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	20-Dec-2018
 */

#ifndef __TOOL_STACK_H
#define __TOOL_STACK_H

#include <assert.h>

/** a fixed size LIFO stack class */
template <class T, int SIZE>
class Stack
{
private:
	T	stack[SIZE];
	int	sp;

public:
	Stack()	: sp(-1) {}

	bool	empty()		{ return sp<0;     }
	bool	full()		{ return sp>=SIZE; }
	bool	one()		{ return sp==0;    }
	int	size()		{ return sp+1;     }
	int	ptr()		{ return sp;       }

	void	push(T x) {
			stack[++sp] = x;
			assert(sp < SIZE);
		}
	void	operator<<(T x)	{ push(x); }

	/** replace the topmost item */
	void	set(T x)	{ assert(sp>=0); stack[sp] = x;      }

	/** peek topmost element */
	T&	peek()		{ assert(sp>=0); return stack[sp];   }
	T&	operator()()	{ return peek();   }
	T	peek(int n)	{ assert(sp-n>=0 and sp-n<SIZE); return stack[sp-n]; }

	/** access any element in stack */
	T	operator[](int n) const	{ return stack[n];  }
	T&	operator[](int n)	{ return stack[n];  }

	/** pop topmost element */
	T	pop()		{ assert(sp>=0); return stack[sp--]; }
}; // class Stack

#endif
